//----------------------------------------//
// billboard.cpp - main file for the billboard stuff
//
//----------------------------------------//

//include the stuff
#include "billboard.h"


//--------------------------------------------------//
// Name: Billboard()
// Desc: constructor - gives default values
//--------------------------------------------------//


Billboard::Billboard()
{
	Device = NULL;
	Texture = NULL;
	VertexBuffer = NULL;
	Size = 0.0f;

}

//--------------------------------------------------//
// Name: ~Billboard()
// Desc: destructor - de-alloctates everything
//--------------------------------------------------//
Billboard::~Billboard()
{

	//release the texture and vertex buffer
	SAFE_RELEASE(Texture);
	SAFE_RELEASE(VertexBuffer);

}

//--------------------------------------------------//
// Name: InitializeBillboard()
// Desc: sets up the billboard for rendering
//--------------------------------------------------//
int Billboard::InitializeBillboard(LPDIRECT3DDEVICE8 device, LPCSTR TexFileName, 
								   float size, D3DXVECTOR3 pos)
{
	//fill in the variables
	Device = device;
	Size = size;
	Position = pos;

	//load the texture
	if(FAILED(D3DXCreateTextureFromFileExA(Device, //device
				TexFileName,			//source name
				D3DX_DEFAULT,	//width
				D3DX_DEFAULT,	//height
				D3DX_DEFAULT,	//# of mipmap levels
				0,				//usage - rendertarget, etc
				D3DFMT_UNKNOWN,	//format
				D3DPOOL_MANAGED,	//mem pool
				D3DX_DEFAULT, //texture filtering
				D3DX_DEFAULT, //mipmap filtering
				0,				//colorkey
				NULL,			//source info
				NULL,			//palatte info
				&Texture)))
	{
		//it failed
		logfile.LogtoFile("BILL - failed to load billboard texture.\n");
		return BILLBOARD_BAD;
	}

	//fill in the vertex buffer
	BILLBOARD_VERTEX vertexarray[4]; 

	//the 4 points - top left, top right, bottom right, bottom left
	vertexarray[0].position = D3DXVECTOR3((-Size/2), (Size/2), 0.0f);
	vertexarray[0].tu = 0.0f;
	vertexarray[0].tv = 0.0f;
	
	vertexarray[1].position = D3DXVECTOR3((Size/2), (Size/2), 0.0f);
	vertexarray[1].tu = 1.0f;
	vertexarray[1].tv = 0.0f;

	vertexarray[2].position = D3DXVECTOR3((Size/2), (-Size/2), 0.0f);
	vertexarray[2].tu = 1.0f;
	vertexarray[2].tv = 1.0f;

	vertexarray[3].position = D3DXVECTOR3((-Size/2), (-Size/2), 0.0f);
	vertexarray[3].tu = 0.0f;
	vertexarray[3].tv = 1.0f;

	//actually create the vertex buffer
	if(FAILED(Device->CreateVertexBuffer(4*sizeof(BILLBOARD_VERTEX), D3DUSAGE_WRITEONLY, 
		D3DFVF_BILLBOARDVERTEX, D3DPOOL_DEFAULT, &VertexBuffer)))
	{
		//uh-oh, it failed!
		logfile.LogtoFile("BILL - failed to create vertex buffer.\n");
		return BILLBOARD_BAD;
	}

	//now fill in the vertex buffer
	VOID* pVertices;
	if(FAILED(VertexBuffer->Lock(0, sizeof(vertexarray), (BYTE**)&pVertices, 0)))
	{
		//hmm, this can't be good!
		logfile.LogtoFile("BILL - failed to lock vertex buffer.\n");
		return BILLBOARD_BAD;
	}

	//copy the vertices to the vertex buffer
	memcpy( pVertices, vertexarray, sizeof(vertexarray) );

	//unlock the vertex buffer
	VertexBuffer->Unlock();


	//must have gone ok
	return BILLBOARD_OK;
}

//--------------------------------------------------//
// Name: RenderBillboard()
// Desc: renders the billboard
//--------------------------------------------------//
int Billboard::RenderBillboard()
{
	D3DXMATRIX matView;
	D3DXMATRIX matTransposed;

	//get the current view matrix
	Device->GetTransform(D3DTS_VIEW, &matView);

	//get the transposed matrix
	D3DXMatrixTranspose(&matTransposed, &matView);

	matTransposed._14 = matTransposed._24 = matTransposed._34 = 0.0f;

	//position the billboard
	matTransposed._41 = Position.x;
	matTransposed._42 = Position.y;
	matTransposed._43 = Position.z;

	// Set as world transformation matrix
	Device->SetTransform(D3DTS_WORLD, &matTransposed);
 
	//render the 2 triangles

	//set the texture stage
	Device->SetTexture(0,Texture);

	//turn alpha blending on
	Device->SetRenderState( D3DRS_ZWRITEENABLE, FALSE );

	Device->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);

	Device->SetRenderState( D3DRS_SRCBLEND, D3DBLEND_SRCCOLOR );
    Device->SetRenderState( D3DRS_DESTBLEND, D3DBLEND_INVSRCCOLOR );


	//render the front 2 triangles
    Device->SetStreamSource(0, VertexBuffer, sizeof(BILLBOARD_VERTEX));
    Device->SetVertexShader(D3DFVF_BILLBOARDVERTEX);
    Device->DrawPrimitive(D3DPT_TRIANGLEFAN, 0, 2);

	//turn alpha blending off
	Device->SetRenderState( D3DRS_ALPHABLENDENABLE, FALSE );
	Device->SetRenderState( D3DRS_ZWRITEENABLE, TRUE );



	//must have gone ok
	return BILLBOARD_OK;
}