//----------------------------------------//
// milkshape.cpp - main file for the milkshape model loader
//
//----------------------------------------//

#include <stdio.h>			//file i/o stuff 
#include "milkshape.h"


//----------------------------------------//
// Name: MODEL()
// Purpose: the constructor - initialize everything
//----------------------------------------//
MODEL::MODEL()
{
	//set everything to either null or 0
	NumGroups = 0;
	PointerGroups = NULL;
	NumMaterials = 0;	
	PointerMaterials = NULL;

}

//----------------------------------------//
// Name: ~MODEL()
// Purpose: the destructor - kills everything
//----------------------------------------//
MODEL::~MODEL()
{
	// If the groups exist
	if(PointerGroups)
	{
		// Release all VB's
		for(int i=0; i<NumGroups; i++)
			if(PointerGroups[i].pVB)
				PointerGroups[i].pVB->Release();

		// Delete the final groups
		delete [] PointerGroups;
		PointerGroups = NULL;
	}

	// If the materials exist
	if(PointerMaterials)
	{
		// Release all textures
		for(int i=0; i<NumMaterials; i++)
		{
			// Release texture numero uno
			if(PointerMaterials[i].pTex1)
				PointerMaterials[i].pTex1->Release();

			// Release texture numero dos
			if(PointerMaterials[i].pTex2)
				PointerMaterials[i].pTex2->Release();
		}

		// Delete the final groups
		delete [] PointerMaterials;
		PointerMaterials = NULL;
	}

}

//----------------------------------------//
// Name: GetPathAndFile()
// Purpose: GetPathAndFile() splits a filename (path included) into a path
// and the filename of the file itself, i.e. (SomeFolder\SomeFile is split
// into "SomeFolder" as the path and "SomeFile" as the file.
//----------------------------------------//
HRESULT MODEL::GetPathAndFile(char * szFilename, char * szPath, char * szFile)
{
	// If any of the parameters was insufficient, fail.
	if(!szFilename ||
		!szPath ||
		!szFile)
	{
		logfile.LogtoFile("MDL - GetPathAndFile passed a bad parameter.\n");
		return MSMODEL_FAIL;
	}

	// Copy the filename to the path buffer
	strcpy(szPath, szFilename);

	// Save a copy of the length of the path
	int iPathLen = strlen(szPath);

	// For each character in the path
	for(int i=0; i<iPathLen; i++)
		// If it's a forward slash
		if(szPath[i] == '/')
			// Change it into a backslash
			szPath[i] = '\\';

	// Find the last occurrence of a backslash
	char * pSlashPos = strrchr(szPath, '\\');

	// If there was a backslash (i.e. a path was specified)
	if(pSlashPos)
	{
		// Clip off the Path string there
		*pSlashPos='\0';

		// Copy what's left over from the path string after that new
		// NULL into the filename-only buffer
		strcpy(szFile, pSlashPos+1);
	}
	// Otherwise
	else
		// Just copy the filename into the filename-only buffer, because
		// there was no path specified.
		strcpy(szFile, szFilename);

	// Return success.
	return MSMODEL_SUCCESS;
}


//----------------------------------------//
// Name: CreateModelFromFile()
// Purpose: do the actual and final loading of the model
//----------------------------------------//
HRESULT MODEL::CreateModelFromFile(LPDIRECT3DDEVICE8 device, char * szFilename, float fScale)
{
	// Separate the filename into the path and the filename-only parts.
	// i.e. if data/ufo.txt was specified, separate it into "data" and "ufo.txt".
	char szPath[MAX_PATH], szFileOnly[MAX_PATH];
	if(GetPathAndFile(szFilename, szPath, szFileOnly) != MSMODEL_SUCCESS)
	{
		logfile.LogtoFile("MDL - GetPathAndFile function failed when trying to load model\n");
		return MSMODEL_FAIL;
	}

	// Save the old working directory
	char szOldCurrDir[MAX_PATH];
	GetCurrentDirectory(MAX_PATH, szOldCurrDir);

	// Switch to the new working directory
	SetCurrentDirectory(szPath);

	//try to create the model and see if it fails
	if(FAILED(LoadModel(device, szFileOnly, fScale)))
	{
		//it messed up trying to load, so return the bad return from the 
		//function
		return MSMODEL_FAIL;
	}

	SetCurrentDirectory(szOldCurrDir);

	//it worked, so return the good return type from the function
	return MSMODEL_SUCCESS;

}

//----------------------------------------//
// Name: LoadModel()
// Purpose: Load() handles all the real loading, and is called 
// by CreateModelFromFile() to do all the loading without having 
// to hand out too much access to its private members.
//----------------------------------------//
HRESULT MODEL::LoadModel(LPDIRECT3DDEVICE8 device, char * szFilename, float fScale)
{
	int iDummy;
	bool bTexturesMissing = false;

	// If anything that was passed is NULL, fail
	if(szFilename == NULL)
	{
		logfile.LogtoFile("MDL - NULL file name passed to LoadModel \n");
		return MSMODEL_FAIL;
	}

	// Try to open the file
	FILE *pFile = fopen(szFilename, "r");

	// If the file couldn't be opened, return failure
	if(pFile == NULL)
	{
		// Close the file & return error!
		logfile.LogtoFile("MDL - LoadModel failed to open file\n");
		fclose(pFile);
		return MSMODEL_FAIL;
	}

	// Read in the title comment
	if(fscanf(pFile, " // MilkShape 3D ASCII ") == EOF)
	{
		// Close the file & return error!
		logfile.LogtoFile("MDL - File loaded by LoadModel was not of the correct file type \n");
		fclose(pFile);
		return MSMODEL_FAIL;
	}

	// Read in the frame info (we really don't load the frames)
	if(fscanf(pFile, " Frames: %d Frame: %d ", &iDummy, &iDummy) != 2)
	{
		// Close the file & return error!
		logfile.LogtoFile("MDL - LoadModel failed to load frame information of model \n");
		fclose(pFile);
		return MSMODEL_FAIL;
	}

	// Read in the mesh count
	if(fscanf(pFile, " Meshes: %d ", &NumGroups) != 1)
	{
		// Close the file & return error!
		logfile.LogtoFile("MDL - LoadModel failed to load mesh count\n");
		fclose(pFile);
		return MSMODEL_FAIL;
	}

	// Allocate memory for all the meshes
	PointerGroups = new FINAL_GROUP[NumGroups];
	ZeroMemory(PointerGroups, sizeof(FINAL_GROUP)*NumGroups);

	// Read in all the meshes
	int iGroup;
	for(iGroup=0; iGroup<NumGroups; iGroup++)
	{
		// Create an intermediate mesh group to load data into
		INTERMEDIATE_GROUP Group;

		// Read in this group's basic data
		// (Name, flags, material)
		if(fscanf(pFile, " \"%[^\"]\" %d %d ",
			      Group.szName,
				  &iDummy,
				  &Group.iMaterial) != 3)
		{
			// Close the file & return error!
			logfile.LogtoFile("MDL - LoadModel failed to read basic group data \n");
			fclose(pFile);
			return MSMODEL_FAIL;
		}

		// Vertex list processing

		// Read in the # of verts
		if(fscanf(pFile, " %d ", &Group.iVerts) != 1)
		{
			// Close the file & return error!
			logfile.LogtoFile("MDL - LoadModel failed to load in number of vertices\n");
			fclose(pFile);
			return MSMODEL_FAIL;
		}

		// Allocate memory for the verts
		Group.pVerts = new INTERMEDIATE_VERT[Group.iVerts];

		// Read in each vert
		// (Flags, x, y, z, u, v, bone index)
		int iVert;
		for(iVert=0; iVert<Group.iVerts; iVert++)
			if(fscanf(pFile, " %d %f %f %f %f %f %d ",
				&iDummy,
				&Group.pVerts[iVert].x,
				&Group.pVerts[iVert].y,
				&Group.pVerts[iVert].z,
				&Group.pVerts[iVert].u,
				&Group.pVerts[iVert].v,
				&iDummy) != 7)
			{
				delete [] Group.pVerts;

				// Close the file & return error!
				logfile.LogtoFile("MDL - LoadModel failed to read in vertices\n");
				fclose(pFile);
				return MSMODEL_FAIL;
			}

		// Normal list processing

		// Read in the # of normals
		if(fscanf(pFile, " %d ", &Group.iNormals) != 1)
		{
			// Close the file & return error!
			logfile.LogtoFile("MDL - LoadModel failed to read in number of normals\n");
			fclose(pFile);
			return MSMODEL_FAIL;
		}
			
		// Allocate memory for the normals
		Group.pNormals = new INTERMEDIATE_NORMAL[Group.iNormals];
			
		// Read in each normal
		// (nx, ny, nz)
		int iNorm;

		for(iNorm=0; iNorm<Group.iNormals; iNorm++)
			if(fscanf(pFile, " %f %f %f ",
				&Group.pNormals[iNorm].nx,
				&Group.pNormals[iNorm].ny,
				&Group.pNormals[iNorm].nz) != 3)
			{
				delete [] Group.pVerts;
				delete [] Group.pNormals;

				// Close the file & return error!
				logfile.LogtoFile("MDL - LoadModel failed to read in normals\n");
				fclose(pFile);
				return MSMODEL_FAIL;
			}
			
		// Face list processing

		// Read in the # of faces
		if(fscanf(pFile, " %d ", &Group.iFaces) != 1)
		{
			// Close the file & return error!
			logfile.LogtoFile("MDL - LoadModel failed to read in number of faces\n");
			fclose(pFile);
			return MSMODEL_FAIL;
		}

		// Allocate memory for the faces
		Group.pFaces = new INTERMEDIATE_FACE[Group.iFaces];

		// Read in each face
		// (flags, a, b, c, na, nb, nc, smoothing group idx)
		int iFace;
		for(iFace=0; iFace<Group.iFaces; iFace++)
			if(fscanf(pFile, " %d %d %d %d %d %d %d %d ",
				&iDummy,
				&Group.pFaces[iFace].a,
				&Group.pFaces[iFace].b,
				&Group.pFaces[iFace].c,
				&Group.pFaces[iFace].na,
				&Group.pFaces[iFace].nb,
				&Group.pFaces[iFace].nc,
				&iDummy) != 8)
			{
				delete [] Group.pVerts;
				delete [] Group.pNormals;
				delete [] Group.pFaces;

				// Close the file & return error!
				fclose(pFile);
				return MSMODEL_FAIL;
			}

		// Set up the properties for this final group
		PointerGroups[iGroup].iMaterial		= Group.iMaterial;
		PointerGroups[iGroup].iPrimitives	= Group.iFaces;

		// Create the vertex buffer for this group
		int iVBSize = Group.iFaces*sizeof(MODELVERTEX)*3;
		if(FAILED(device->CreateVertexBuffer(iVBSize,
                                              D3DUSAGE_WRITEONLY, D3DFVF_MODELVERTEX,
                                              D3DPOOL_DEFAULT, 
											  &PointerGroups[iGroup].pVB)))
		{
			delete [] Group.pVerts;
			delete [] Group.pNormals;
			delete [] Group.pFaces;

			// Close the file & return error!
			logfile.LogtoFile("MDL - LoadModel failed to create a vertex buffer\n");
			fclose(pFile);
			return MSMODEL_FAIL;
		}

		// Create all the verts to add to the VB
		MODELVERTEX * pVerts = new MODELVERTEX[Group.iFaces*3];
		for(iFace=0; iFace<Group.iFaces; iFace++)
		{
			// For each vert on this face
			for(int i=0; i<3; i++)
			{
				// Select the correct set of indexes based on the current vert
				// we're looking at (#1, #2 or #3) of this face
				int iVert, iNorm;
				if(i == 0)
				{
					iVert	= Group.pFaces[iFace].a;
					iNorm	= Group.pFaces[iFace].na;
				}
				else if(i == 1)
				{
					iVert	= Group.pFaces[iFace].b;
					iNorm	= Group.pFaces[iFace].nb;
				}
				else if(i == 2)
				{
					iVert	= Group.pFaces[iFace].c;
					iNorm	= Group.pFaces[iFace].nc;
				}

				// Get the index of the vertex we're editing
				int iOutVert = iFace*3+i;

				// Set this vertex's data
				pVerts[iOutVert].position.x		=	Group.pVerts[iVert].x * fScale;
				pVerts[iOutVert].position.y		=	Group.pVerts[iVert].y * fScale;
				pVerts[iOutVert].position.z		=	Group.pVerts[iVert].z * fScale;
				pVerts[iOutVert].tu		=	Group.pVerts[iVert].u;
				pVerts[iOutVert].tv		=	Group.pVerts[iVert].v;
				pVerts[iOutVert].normal.x		=	Group.pNormals[iNorm].nx;
				pVerts[iOutVert].normal.y		=	Group.pNormals[iNorm].ny;
				pVerts[iOutVert].normal.z		=	Group.pNormals[iNorm].nz;

			} //end for
		} //end bigger for

		// Write all this data to the VB
		VOID* pVertices;
		if(FAILED(PointerGroups[iGroup].pVB->Lock(0, iVBSize, (BYTE**)&pVertices, 0)))
			return MSMODEL_FAIL;
		memcpy( pVertices, pVerts, iVBSize);
		PointerGroups[iGroup].pVB->Unlock();

		// Delete the intermediate group
		delete [] Group.pVerts;
		delete [] Group.pNormals;
		delete [] Group.pFaces;

	} //end for- number of groups

	// Read in the material count
	if(fscanf(pFile, " Materials: %d ", &NumMaterials) != 1)
	{
		// Close the file & return error!
		logfile.LogtoFile("MDL - LoadModel failed to read in material count\n");
		fclose(pFile);
		return MSMODEL_FAIL;
	}

	// Allocate memory for all the materials
	PointerMaterials = new FINAL_MATERIAL[NumMaterials];
	ZeroMemory(PointerMaterials, sizeof(FINAL_MATERIAL)*NumMaterials);

	// Read in the actual materials
	int iMaterial;
	for(iMaterial=0; iMaterial < NumMaterials; iMaterial++)
	{
		char szTempBuf[256]; // Temporary dummy buffer
		char szTex1[256], szTex2[256];	// Texture filenames

		// Get a ptr to the material for simpler references
		FINAL_MATERIAL * pMat = &PointerMaterials[iMaterial];

		//this is one motha of an fscanf()!
		if(fscanf(pFile, " \"%[^\"]\" %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f %f ",
			szTempBuf,
			&pMat->Mat.Ambient.r,	&pMat->Mat.Ambient.g,	&pMat->Mat.Ambient.b,	&pMat->Mat.Ambient.a,
			&pMat->Mat.Diffuse.r,	&pMat->Mat.Diffuse.g,	&pMat->Mat.Diffuse.b,	&pMat->Mat.Diffuse.a,
			&pMat->Mat.Specular.r,	&pMat->Mat.Specular.g,	&pMat->Mat.Specular.b,	&pMat->Mat.Specular.a,
			&pMat->Mat.Emissive.r,	&pMat->Mat.Emissive.g,	&pMat->Mat.Emissive.b,	&pMat->Mat.Emissive.a,
			&pMat->Mat.Power,
			&pMat->fAlpha) != 19)
		{
			// Close the file & return error!
			logfile.LogtoFile("MDL - LoadModel failed to read in material information\n");
			fclose(pFile);
			return MSMODEL_FAIL;
		}

		// If it's reflective (first char '@') set up the flag
		if(szTempBuf[0] == '@')
			pMat->bRef = true;
		else
			pMat->bRef = false;

		if(szTempBuf[1] == '*')
			pMat->alphablend = true;
		else
			pMat->alphablend = false;

		// Read in the next two lines (texture names)

		// First line... read in the source filename for Texture 1.
		fgets(szTempBuf, 256, pFile);
		if(szTempBuf == NULL)
		{
			// Close the file & return error!
			logfile.LogtoFile("MDL - LoadModel failed to read in texture file info for tex 1\n");
			fclose(pFile);
			return MSMODEL_FAIL;
		}

		if(szTempBuf[0] == '\"' && szTempBuf[1] == '\"')
			szTex1[0] = '\0';
		else if(sscanf(szTempBuf, " \"%[^\"]\" ", szTex1) != 1)
		{
			// Close the file & return error!
			logfile.LogtoFile("MDL - LoadModel failed to read texture file name info for tex 1\n");
			fclose(pFile);
			return MSMODEL_FAIL;
		}

		// First line... read in the source filename for Texture 2.
		fgets(szTempBuf, 256, pFile);

		if(szTempBuf == NULL)
		{
			// Close the file & return error!
			logfile.LogtoFile("MDL - LoadModel failed to read in texture file name info for tex 2\n");
			fclose(pFile);
			return MSMODEL_FAIL;
		}

		if(szTempBuf[0] == '\"' && szTempBuf[1] == '\"')
			szTex2[0] = '\0';
		else if(sscanf(szTempBuf, " \"%[^\"]\" ", szTex2) != 1)
		{
			// Close the file & return error!
			logfile.LogtoFile("MDL - LoadModel failed to read in texture file name info for tex 2\n");
			fclose(pFile);
			return MSMODEL_FAIL;
		}

		// If there is a texture #1
		if(strlen(szTex1) != 0)
		{
			// Try to load it
			if(FAILED(D3DXCreateTextureFromFileExA(device, //device
				szTex1,			//source name
				D3DX_DEFAULT,	//width
				D3DX_DEFAULT,	//height
				D3DX_DEFAULT,	//# of mipmap levels
				0,				//usage - rendertarget, etc
				D3DFMT_UNKNOWN,	//format
				D3DPOOL_MANAGED,	//mem pool
				D3DX_FILTER_LINEAR, //texture filtering
				D3DX_FILTER_LINEAR, //mipmap filtering
				0,				//colorkey
				NULL,			//source info
				NULL,			//palatte info
				&pMat->pTex1)))
			{
				//its missing textures
				bTexturesMissing = true;
			}
		}

		// If there is a texture #2
		if(strlen(szTex2) != 0)
		{
			// Try to load it
			if(FAILED(D3DXCreateTextureFromFileExA(device, //device
				szTex1,			//source name
				D3DX_DEFAULT,	//width
				D3DX_DEFAULT,	//height
				D3DX_DEFAULT,	//# of mipmap levels 
				0,				//usage - rendertarget, etc
				D3DFMT_UNKNOWN,	//format
				D3DPOOL_MANAGED,	//mem pool
				D3DX_FILTER_LINEAR, //texture filtering
				D3DX_FILTER_LINEAR, //mipmap filtering
				0,				//colorkey
				NULL,			//source info
				NULL,			//palatte info
				&pMat->pTex2)))
			{
				//its missing textures
				bTexturesMissing = true;
			}
		} //end if
	} //end for materials

	// Close the file & return OK
	fclose(pFile);

	// Depening if textures were missing, return different success flags
	if(!bTexturesMissing)
		return MSMODEL_SUCCESS;
	else
		return MSMODEL_FAIL;

}

//----------------------------------------//
// Name: RenderModel()
// Purpose: renders the model. duh!
//----------------------------------------//
HRESULT MODEL::RenderModel(LPDIRECT3DDEVICE8 device)
{
	// Set the vertex shader to the model vertex one
	if(device->SetVertexShader(D3DFVF_MODELVERTEX)==D3DERR_INVALIDCALL)
	{
		//do the usual failure stuff
		logfile.LogtoFile("MLD - Failed to set Vertex Shader for model rendering\n");
		return MSMODEL_FAIL;
	}

	// Render each VB
	for(int i=0; i<NumGroups; i++)
	{
		// If this is a valid material index
		if(PointerGroups[i].iMaterial < NumMaterials &&
		   PointerGroups[i].iMaterial >= 0)
		{
			// Get a ptr. to the material of this VB for faster referencing
			FINAL_MATERIAL * pMat = &PointerMaterials[PointerGroups[i].iMaterial];

			// If it has a reflective-style texture
			if(pMat->bRef)
			{
				device->SetTextureStageState( 0, D3DTSS_TEXTURETRANSFORMFLAGS, D3DTTFF_COUNT2 );
				device->SetTextureStageState( 0, D3DTSS_TEXCOORDINDEX, D3DTSS_TCI_CAMERASPACENORMAL );
			} else {
				// Just use the normal texture coordinates
				device->SetTextureStageState( 0, D3DTSS_TEXCOORDINDEX, 
													D3DTSS_TCI_PASSTHRU);
			}

			if(pMat->alphablend)
			{

				device->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
				device->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_ONE);
				device->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);
			}

			// Otherwise
			

			// Select the material properties into use
			device->SetMaterial(&pMat->Mat);

			// Select the texture into use
			device->SetTexture(0, pMat->pTex1);
		}
		// Otherwise if this is an invalid material
		else
		{
			// Just use a default material
			device->SetMaterial(&g_GenericMaterial);

			// Use no texturing either
			device->SetTexture(0, NULL);
		}

		// Set this vertex buffer as the vertex data stream source
		device->SetStreamSource(0, PointerGroups[i].pVB, sizeof(MODELVERTEX));

		// Draw the correct number of triangles from that VB
		if(FAILED(device->DrawPrimitive(D3DPT_TRIANGLELIST, 0, 
										PointerGroups[i].iPrimitives)))
		{
			//do the usual failure stuff
			logfile.LogtoFile("MDL - Failed draw vertices for model\n");
			return MSMODEL_FAIL;
		}

		device->SetTextureStageState( 0, D3DTSS_TEXCOORDINDEX, D3DTSS_TCI_PASSTHRU);
		device->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);

	}

	//it must have done it so return ok
	return MSMODEL_SUCCESS;
}

