//----------------------------------------//
// enginerender.cpp - main file for the rendering
//		functions of the engine
//
//----------------------------------------//

#include "engine.h"
#include <mmsystem.h>	//for timeGetTime and some other windows stuff

//----------------------------------------//
// Name: RenderMainScene()
// Purpose: render the main scene of the game - eg level data
// Note: the backbuffer is presented no matter the gamestate
//----------------------------------------//
int ENGINE::RenderMainScene()
{
	//clear the backbuffer and z buffer with the color specified in the startup file
	if(d3d.lpd3ddevice->Clear( 0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER,
				D3DCOLOR_XRGB(0, 0, 128), 1.0f, 0 ) != D3D_OK)
	{
		logfile.LogtoFile("ENG - Failed to clear backbuffer and z buffer\n");
		return ENGINE_BAD;
	}

	//begin rendering the scene
	d3d.lpd3ddevice->BeginScene();

	//try spitting out the fps
	static char buffer[20];
	static float fps = 0;

	//test out the camera updating function
	Camera.UpdateCamera(d3d.lpd3ddevice, timer.Delta);

	// Rotate the world matrix by those rotation values.
	D3DXMATRIX matWorld;

	//render the skybox without lighting and not moving
	D3DXMatrixIdentity(&matWorld);
	d3d.lpd3ddevice->SetTransform(D3DTS_WORLD, &matWorld);
	d3d.lpd3ddevice->SetRenderState(D3DRS_LIGHTING, FALSE);
	Skybox.RenderModel(d3d.lpd3ddevice);

	//render the cloud box without lighting but moving
	D3DXMatrixRotationYawPitchRoll(&matWorld, 0.0f, timeGetTime()/20000.0f, 0.0f);
	d3d.lpd3ddevice->SetTransform(D3DTS_WORLD, &matWorld);
	Cloudbox.RenderModel(d3d.lpd3ddevice);

	//turn lighting back on
	d3d.lpd3ddevice->SetRenderState(D3DRS_LIGHTING, TRUE);

	//render the level
	D3DXMatrixIdentity(&matWorld);
	d3d.lpd3ddevice->SetTransform(D3DTS_WORLD, &matWorld);
	Level.RenderModel(d3d.lpd3ddevice);

	//try thatt billboard
	d3d.lpd3ddevice->SetRenderState(D3DRS_LIGHTING, FALSE);
	if(TestBillboard1.RenderBillboard() != BILLBOARD_OK)
	{
		logfile.LogtoFile("ENG - failed to draw billboard.\n");
		return ENGINE_BAD;
	}

	if(TestBillboard2.RenderBillboard() != BILLBOARD_OK)
	{
		logfile.LogtoFile("ENG - failed to draw billboard.\n");
		return ENGINE_BAD;
	}
	d3d.lpd3ddevice->SetRenderState(D3DRS_LIGHTING, TRUE);

	
	//output the frames per second if its supposed to be
	if(ShowFPS == 1)
	{
		if(timer.CheckOneSecondTimer()==TIMER_OK)
		{
			fps = 1.0f/timer.Delta;
			sprintf(buffer,"FPS:%.000f",fps);
			font.Draw(0.0f, 0.0f, buffer);
		} else {
			font.Draw(0.0f, 0.0f, buffer);
		}
	}

	//finish rendering the scene
	d3d.lpd3ddevice->EndScene();

	//very last thing, present the backbuffer to the front bufffer
    d3d.lpd3ddevice->Present( NULL, NULL, NULL, NULL );


	//return ok
	return ENGINE_OK;
}


//----------------------------------------//
// Name: RenderMainMenu()
// Purpose: renders the main menu for the game
//----------------------------------------//
int ENGINE::RenderMainMenu()
{
		//clear the backbuffer and z buffer with the color specified in the startup file
	if(d3d.lpd3ddevice->Clear( 0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER,
				D3DCOLOR_XRGB(0, 0, 0), 1.0f, 0 ) != D3D_OK)
	{
		logfile.LogtoFile("ENG - Failed to clear backbuffer and z buffer\n");
		return ENGINE_BAD;
	}

	//begin rendering the scene
	d3d.lpd3ddevice->BeginScene();

	//try spitting out the fps
	static char buffer[20];
	static float fps = 0;

	D3DXMATRIX matView;
	D3DXMatrixLookAtLH( &matView, &D3DXVECTOR3( 0.0f, 2.5f, 5.0f),
								  &D3DXVECTOR3( 0.0f, 1.0f, 0.0f ),
								  &D3DXVECTOR3( 0.0f, 1.0f, 0.0f ) );
	d3d.lpd3ddevice->SetTransform( D3DTS_VIEW, &matView );


	// Rotate the world matrix by those rotation values.
	D3DXMATRIX matWorld;

	//draw that cool 4E3 logo
	D3DXMatrixRotationYawPitchRoll(&matWorld, timeGetTime()/1000.0f, 0.0f, 0.0f);
	d3d.lpd3ddevice->SetTransform(D3DTS_WORLD, &matWorld);

	//turn lighting on/off to make it look cool
	d3d.lpd3ddevice->SetRenderState(D3DRS_LIGHTING, FALSE);
	FourE3.RenderModel(d3d.lpd3ddevice);
	d3d.lpd3ddevice->SetRenderState(D3DRS_LIGHTING, TRUE);



	//draw the sprite
	d3d.lpd3ddevice->SetRenderState(D3DRS_ALPHABLENDENABLE, TRUE);
	d3d.lpd3ddevice->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
	d3d.lpd3ddevice->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);

	MenuSprite1.DrawSprite(D3DXVECTOR2(0.0f, 0.0f));
	MenuSprite2.DrawSprite(D3DXVECTOR2(0.0f, 128.0f));

	d3d.lpd3ddevice->SetRenderState(D3DRS_ALPHABLENDENABLE, FALSE);


	//output the frames per second if its supposed to be
	if(ShowFPS == 1)
	{
		font.SetFontColor(128, 255, 255, 255);

		if(timer.CheckOneSecondTimer()==TIMER_OK)
		{
			fps = 1.0f/timer.Delta;
			sprintf(buffer,"FPS:%.000f",fps);
			font.Draw(0.0f, 0.0f, buffer);
		} else {
			font.Draw(0.0f, 0.0f, buffer);
		}
	}

	/*char mousebuffer[20];
	sprintf(buffer,"Mouse X % Y %", directinput.MouseX, directinput.MouseY);
	font.Draw(0.0f, 16.0f, mousebuffer);
*/
	//finish rendering the scene
	d3d.lpd3ddevice->EndScene();

	//very last thing, present the backbuffer to the front bufffer
    d3d.lpd3ddevice->Present( NULL, NULL, NULL, NULL );




	//return ok
	return ENGINE_OK;
}
